/***************************************************************************************************************:')

CoordSpace2Img.h

Coordinate space handling in images.

Fabrice Le Bars

Created: 2009-01-07

Version status: Not finished

***************************************************************************************************************:)*/

#ifndef COORDSPACE2IMG_H
#define COORDSPACE2IMG_H

#include "CoordSpace.h"

/*
Structure that enables the use of a coordinate space in an image instead of 
using its lines and rows.
*/
struct _COORDSPACE2IMG
{
	COORDSPACE cs;
	UINT width;
	UINT height;
	double XJRatio;
	double YIRatio;
	double JXRatio;
	double IYRatio;
};
typedef struct _COORDSPACE2IMG COORDSPACE2IMG;

/*
Initialize a structure COORDSPACE2IMG.
See the description of the COORDSPACE2IMG structure for more information.

COORDSPACE2IMG* pCS2Img : (INOUT) Valid pointer to the structure.
COORDSPACE* pCS : (IN) Coordinate space to use.
UINT width : (IN) Width of the picture.
UINT height : (IN) Height of the picture.

Return : EXIT_SUCCESS or EXIT_FAILURE if there is an error.
*/
__inline int InitCS2Img(COORDSPACE2IMG* pCS2Img, COORDSPACE* pCS, UINT width, UINT height)
{
	// x = xMin -> j = 0
	// x = xMax -> j = width-1
	// y = yMin -> i = height-1
	// y = yMax -> i = 0

	pCS2Img->cs = *pCS;

	pCS2Img->width = width;
	pCS2Img->height = height;

	pCS2Img->XJRatio = (pCS2Img->cs.xMax - pCS2Img->cs.xMin) / (double)(pCS2Img->width - 1);
	pCS2Img->YIRatio = (pCS2Img->cs.yMin - pCS2Img->cs.yMax) / (double)(pCS2Img->height - 1);

	pCS2Img->JXRatio = 1.0/pCS2Img->XJRatio;
	pCS2Img->IYRatio = 1.0/pCS2Img->YIRatio;

	return EXIT_SUCCESS;
}

__inline int XYCS2IJImg(COORDSPACE2IMG* pCS2Img, double x, double y, UINT* pI, UINT* pJ)
{
	*pI = (UINT)((y - pCS2Img->cs.yMax) * pCS2Img->IYRatio);
	*pJ = (UINT)((x - pCS2Img->cs.xMin) * pCS2Img->JXRatio);

	return EXIT_SUCCESS;
}

__inline int IJImg2XYCS(COORDSPACE2IMG* pCS2Img, UINT i, UINT j, double* pX, double* pY)
{
	*pX = pCS2Img->XJRatio * (double)j + pCS2Img->cs.xMin;
	*pY = pCS2Img->YIRatio * (double)i + pCS2Img->cs.yMax;

	return EXIT_SUCCESS;
}

__inline int GetCSPixelSize(COORDSPACE2IMG* pCS2Img, double* pSizeX, double* pSizeY)
{
	*pSizeX = fabs(pCS2Img->XJRatio);
	*pSizeY = fabs(pCS2Img->YIRatio);

	return EXIT_SUCCESS;
}

#endif // COORDSPACE2IMG_H
