//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MainFormUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
extern int mode;
extern int NbComponentsAnd1;
extern vector<box> pCutOut;
extern vector<interval> dminIntervals;
extern double dmins[300][300];
//---------------------------------------------------------------------------

__fastcall TMainForm::TMainForm(TComponent* Owner)
: TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormCreate(TObject *Sender)
{
	for (int i = 0; i < 300; i++)
	{
		for (int j = 0; j < 300; j++)
		{
			dmins[j][i] = 0;
		}
	}

	switch (mode)
	{
	case ACKERMANN:
		AckermannRadioButton->Checked = true;
		InitMainImageAckermann();
		break;
	case RAYTRACING:
		RayTracingRadioButton->Checked = true;
		InitMainImageRayTracing();
		break;
	default:
		break;
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::InitMainImageAckermann(void)
{
	MainImageRepere = repere(MainImage->Canvas,
		X_INF_ACKERMANN,X_SUP_ACKERMANN,
		Y_INF_ACKERMANN,Y_SUP_ACKERMANN);

	MainImage->Picture->LoadFromFile("Ackermann.bmp");

	pCutOut.clear();
	dminIntervals.clear();
        Label1->Caption = "delta = 0 -> black, delta = 1 -> white in parameters space";
        Label2->Caption = "r(delta)";
        Label3->Caption = "Impulse response";
        Label4->Caption = "Poles in complex plane";
        Label5->Caption = "Characteristic polynomial";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::InitMainImageRayTracing(void)
{
	MainImageRepere = repere(MainImage->Canvas,
		X_INF_RAYTRACING,X_SUP_RAYTRACING,
		Y_INF_RAYTRACING,Y_SUP_RAYTRACING);

	MainImage->Picture->LoadFromFile("Raytracing.bmp");

	pCutOut.clear();
	dminIntervals.clear();
        Label1->Caption = "View of 2 spheres";
        Label2->Caption = "g(d) for a given pixel";
        Label3->Caption = "";
        Label4->Caption = "";
        Label5->Caption = "";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::RayTracingRadioButtonClick(TObject *Sender)
{
	mode = RAYTRACING;
	NbComponentsAnd1 = NB_COMPONENTS_FBOX_AND_1;
	InitMainImageRayTracing();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::AckermannRadioButtonClick(TObject *Sender)
{
	mode = ACKERMANN;
	NbComponentsAnd1 = NB_COMPONENTS_RBOX_AND_1;
	InitMainImageAckermann();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::OpenFileSubMenuClick(TObject *Sender)
{
	switch (mode)
	{
	case ACKERMANN:
		MainImage->Picture->LoadFromFile("Ackermann.bmp");
		break;
	case RAYTRACING:
		MainImage->Picture->LoadFromFile("Raytracing.bmp");
		break;
	default:
		break;
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SaveFileSubMenuClick(TObject *Sender)
{
	switch (mode)
	{
	case ACKERMANN:
		MainImage->Picture->SaveToFile("Ackermann.bmp");
		break;
	case RAYTRACING:
		MainImage->Picture->SaveToFile("Raytracing.bmp");
		break;
	default:
		break;
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComputeDrawComputeSubMenuClick(TObject *Sender)
{
	box P0;

	switch (mode)
	{
	case ACKERMANN:
		P0 = box(
			interval(X_INF_ACKERMANN,X_SUP_ACKERMANN), 
			interval(Y_INF_ACKERMANN,Y_SUP_ACKERMANN)
			);
		break;

	case RAYTRACING:
		P0 = box(
			interval(X_INF_RAYTRACING,X_SUP_RAYTRACING), 
			interval(Y_INF_RAYTRACING,Y_SUP_RAYTRACING)
			);
		break;

	default:
		break;
	}

	MainImageRepere = repere(MainImage->Canvas,P0[1].inf,P0[1].sup,P0[2].inf,P0[2].sup);
	MainImageRepere.DrawBox(P0,clBlack,clBlack,bsSolid);

	clock_t start;
	clock_t end;
	double duration;

	start = clock();

	// Launch the algorithm
	SIVIApile(MainImageRepere, P0);

	end = clock();

	duration = (double)((end - start)/CLK_TCK);

	TimeElapsedLabel->Caption = "Time elapsed : " + FloatToStrF(duration, ffGeneral, 4, 4) + "s";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComputeDrawProgressComputeSubMenuClick(
	TObject *Sender)
{
	box P0;

	switch (mode)
	{
	case ACKERMANN:
		P0 = box(
			interval(X_INF_ACKERMANN,X_SUP_ACKERMANN),
			interval(Y_INF_ACKERMANN,Y_SUP_ACKERMANN)
			);
		break;

	case RAYTRACING:
		P0 = box(
			interval(X_INF_RAYTRACING,X_SUP_RAYTRACING),
			interval(Y_INF_RAYTRACING,Y_SUP_RAYTRACING)
			);
		break;

	default:
		break;
	}

	MainImageRepere = repere(MainImage->Canvas,P0[1].inf,P0[1].sup,P0[2].inf,P0[2].sup);
	MainImageRepere.DrawBox(P0,clBlack,clBlack,bsSolid);

	clock_t start;
	clock_t end;
	double duration;

	start = clock();

	// Launch the algorithm
	SIVIApileProgress(MainImageRepere, P0, MainImage);

	end = clock();

	duration = (double)((end - start)/CLK_TCK);

	TimeElapsedLabel->Caption = "Time elapsed : " + FloatToStrF(duration, ffGeneral, 4, 4) + "s";
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComputeDrawBoxesComputeSubMenuClick(
	TObject *Sender)
{
	box P0;

	switch (mode)
	{
	case ACKERMANN:
		P0 = box(
			interval(X_INF_ACKERMANN,X_SUP_ACKERMANN),
			interval(Y_INF_ACKERMANN,Y_SUP_ACKERMANN)
			);
		break;

	case RAYTRACING:
		P0 = box(
			interval(X_INF_RAYTRACING,X_SUP_RAYTRACING),
			interval(Y_INF_RAYTRACING,Y_SUP_RAYTRACING)
			);
		break;

	default:
		break;
	}

	MainImageRepere = repere(MainImage->Canvas,P0[1].inf,P0[1].sup,P0[2].inf,P0[2].sup);
	MainImageRepere.DrawBox(P0,clBlack,clBlack,bsSolid);

	pCutOut.clear();
	dminIntervals.clear();

	clock_t start;
	clock_t end;
	double duration;

	start = clock();

	// Launch the algorithm
	SIVIApileProgressBoxes(MainImageRepere, P0, MainImage);

	end = clock();

	duration = (double)((end - start)/CLK_TCK);

	TimeElapsedLabel->Caption = "Time elapsed : " + FloatToStrF(duration, ffGeneral, 4, 4) + "s";

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::MainImageMouseMove(TObject *Sender,
											  TShiftState Shift, int X, int Y)
{
	double px = 0.0;
	double py = 0.0;


	MainImageRepere.IJImg2XYCS(Y, X, px, py);

	// Displays in the status bar the current coordinates
	MainFormStatusBar->Panels->Items[0]->Text = \
		"(pi;pj)=(" + IntToStr(X) + ";" + IntToStr(Y) + ")" + \
		"\t(px;py)=(" + FloatToStrF(px, ffGeneral, 2, 2) + ";" + FloatToStrF(py, ffGeneral, 2, 2) + ")";

	DrawFcns(X, Y, px, py);
	DrawImpulseResponse(X, Y, px, py);
	DrawPoles(X, Y, px, py);
	DrawPoly(X, Y, px, py);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DrawFcns(int X, int Y, double px, double py)
{
	box b;

	switch (mode)
	{
	case ACKERMANN:
		b = box(interval(D_INF_ACKERMANN,D_SUP_ACKERMANN), interval(-20,20));
		break;

	case RAYTRACING:
		b = box(interval(D_INF_RAYTRACING,D_SUP_RAYTRACING-5), interval(-1,10));
		break;

	default:
		break;
	}

	repere R = repere(FcnsPaintBox->Canvas,b[1].inf,b[1].sup,b[2].inf,b[2].sup);
	R.DrawBox(b,clWhite,clWhite,bsSolid);
	R.DrawLine(b[1].inf,0,b[1].sup,0,clBlack,(TPenStyle)0,1);
	R.DrawLine(0,b[2].inf,0,b[2].sup,clBlack,(TPenStyle)0,1);

	double sizex = 0;
	double sizey = 0;

	R.GetCSPixelSize(sizex, sizey);

	double pxSize = min(sizex, sizey);
	int red = 255;
	int green = 0;
	int blue = 0;

	box P = box(interval(px),interval(py));

	for (double d=R.interval_x.inf; d<R.interval_x.sup; d+=pxSize)
	{
		double g_min = b[2].sup+1;

		for (int i=1; i<NbComponentsAnd1; i++)
		{
			//			red = ((255*((i+1)%3)/2)/(max(0,i-3)+1));
			//			green = ((255*((i+0)%3)/2)/(max(0,i-3)+1));
			//			blue = ((255*((i+2)%3)/2)/(max(0,i-3)+1));

			g_min = min(g_min, gbox(P, interval(d))[i].inf);
		}

		R.DrawPixel(
			d, 
			g_min, 
			(TColor)RGB(red, green, blue)
			);
	}

	if ((!dminIntervals.empty())&&(!pCutOut.empty()))
	{
		for (unsigned int i = 0; i < pCutOut.size(); i++)
		{
			if (In(P, pCutOut[i]) == itrue)
			{
				R.DrawLine(dminIntervals[i].inf,0,dminIntervals[i].sup,0,clBlue,(TPenStyle)0,4);
				break;
			}
		}
	}
	
	if (dmins[X][Y] > 0)
	{
		R.DrawLine(
			dmins[X][Y], -0.5,
			dmins[X][Y], 0.5,
			clBlack, (TPenStyle)0, 4
			);
	}
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DrawImpulseResponse(int X, int Y, double px, double py)
{
	switch (mode)
	{
	case ACKERMANN:
		{
			box b3 = box(interval(0,20), interval(-2,2));
			repere R3 = repere(ImpulseResponsePaintBox->Canvas,b3[1].inf,b3[1].sup,b3[2].inf,b3[2].sup);
			R3.DrawBox(b3,clWhite,clWhite,bsSolid);
			R3.DrawLine(b3[1].inf,0,b3[1].sup,0,clBlack,(TPenStyle)0,1);
			//R3.DrawLine(0,b3[2].inf,0,b3[2].sup,clBlack,(TPenStyle)0,1);

			double a3x = (R3.interval_x.sup - R3.interval_x.inf)/ImpulseResponsePaintBox->Canvas->ClipRect.Width();
			double a3y = (R3.interval_y.sup - R3.interval_y.inf)/ImpulseResponsePaintBox->Canvas->ClipRect.Height();
			double b3x = R3.interval_x.inf/a3x;
			double b3y = R3.interval_y.sup/a3y;

			double dt = 0.001;
			double x1next = 0;
			double x2next = 0;
			double x3next = 0;
			double x1 = 1;
			double x2 = 0;
			double x3 = 0;
			double u = 0;

			for (double t=R3.interval_x.inf; t<R3.interval_x.sup; t+=dt)
			{
				x1next = x2 * dt + x1;
				x2next = x3 * dt + x2;
				x3next = (-(px+py+2)*x3-(px+py+2)*x2-(2*px*py+6*px+6*py+2+sqr(sigma))*x1+u) * dt + x3;

				x1 = x1next;
				x2 = x2next;
				x3 = x3next;

				if ((x1 > 1000)||(x2 > 1000)||(x3 > 1000))
				{
					break;
				}

				double p3x = t;
				double p3y = x1;

				int pi = (int)(p3x/a3x-b3x);
				int pj = (int)(-p3y/a3y+b3y);

				ImpulseResponsePaintBox->Canvas->Pixels[pi][pj] = (TColor)clRed;
			}
		}
		break;

	case RAYTRACING:
		{
			box b3 = box(interval(0,20), interval(-2,2));
			repere R3 = repere(ImpulseResponsePaintBox->Canvas,b3[1].inf,b3[1].sup,b3[2].inf,b3[2].sup);
			R3.DrawBox(b3,clBtnFace,clBtnFace,bsSolid);
		}
		break;

	default:
		break;
	}

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DrawPoles(int X, int Y, double px, double py)
{

	switch (mode)
	{
	case ACKERMANN:
		{
			box b4 = box(interval(-2,2), interval(-2,2));
			repere R4 = repere(PolesPaintBox->Canvas,b4[1].inf,b4[1].sup,b4[2].inf,b4[2].sup);
			R4.DrawBox(b4,clWhite,clWhite,bsSolid);
			R4.DrawLine(b4[1].inf,0,b4[1].sup,0,clBlack,(TPenStyle)0,1);
			R4.DrawLine(0,b4[2].inf,0,b4[2].sup,clBlack,(TPenStyle)0,1);

			double a = px+py+2;
			double b = px+py+2;
			double c = 2*px*py+6*px+6*py+2+sqr(sigma);

			CvMat* pCoeffs;
			CvMat* pRoots;

			pCoeffs = cvCreateMat(4, 1, CV_64FC1);
			pRoots = cvCreateMat(3, 1, CV_64FC2);

			pCoeffs->data.db[0] = c;
			pCoeffs->data.db[1] = b;
			pCoeffs->data.db[2] = a;
			pCoeffs->data.db[3] = 1;

			cvSolvePoly(pCoeffs, pRoots, 10000, 10);

			double maxPole = pRoots->data.db[0];

			for(int i = 0; i < 3; i++ )
			{
				if (pRoots->data.db[i*2] >= 0)
				{
					R4.DrawCircle(pRoots->data.db[i*2],pRoots->data.db[i*2+1],0.1,clRed,clRed,0.1);
				}
				else
				{
					R4.DrawCircle(pRoots->data.db[i*2],pRoots->data.db[i*2+1],0.1,clGreen,clGreen,0.1);
				}
				maxPole = max(maxPole,pRoots->data.db[i*2]);
			}

			if (maxPole >= 0)
			{
				R4.DrawLine(maxPole,b4[2].inf,maxPole,b4[2].sup,clBlue,(TPenStyle)0,1);
				R4.DrawLine(0,1,maxPole,1,clRed,(TPenStyle)0,1);
				R4.DrawLine(0,1,0.1,1.1,clRed,(TPenStyle)0,1);
				R4.DrawLine(0,1,0.1,0.9,clRed,(TPenStyle)0,1);
			}
			else
			{
				R4.DrawLine(maxPole,b4[2].inf,maxPole,b4[2].sup,clBlue,(TPenStyle)0,1);
				R4.DrawLine(0,1,maxPole,1,clGreen,(TPenStyle)0,1);
				R4.DrawLine(0,1,-0.1,1.1,clGreen,(TPenStyle)0,1);
				R4.DrawLine(0,1,-0.1,0.9,clGreen,(TPenStyle)0,1);
			}

			cvReleaseMat(&pCoeffs);
			cvReleaseMat(&pRoots);
		}
		break;

	case RAYTRACING:
		{
			box b4 = box(interval(-10,10), interval(-10,10));
			repere R4 = repere(PolesPaintBox->Canvas,b4[1].inf,b4[1].sup,b4[2].inf,b4[2].sup);
			R4.DrawBox(b4,clBtnFace,clBtnFace,bsSolid);
		}
		break;

	default:
		break;
	}


}
//---------------------------------------------------------------------------

void __fastcall TMainForm::DrawPoly(int X, int Y, double px, double py)
{

	switch (mode)
	{
	case ACKERMANN:
		{
			box b5 = box(interval(-10,10), interval(-100,100));
			repere R5 = repere(PolyPaintBox->Canvas,b5[1].inf,b5[1].sup,b5[2].inf,b5[2].sup);
			R5.DrawBox(b5,clWhite,clWhite,bsSolid);
			R5.DrawLine(b5[1].inf,0,b5[1].sup,0,clBlack,(TPenStyle)0,1);
			R5.DrawLine(0,b5[2].inf,0,b5[2].sup,clBlack,(TPenStyle)0,1);

			double a5x = (R5.interval_x.sup - R5.interval_x.inf)/PolyPaintBox->Canvas->ClipRect.Width();
			double a5y = (R5.interval_y.sup - R5.interval_y.inf)/PolyPaintBox->Canvas->ClipRect.Height();
			double b5x = R5.interval_x.inf/a5x;
			double b5y = R5.interval_y.sup/a5y;

			for (double s=R5.interval_x.inf; s<R5.interval_x.sup; s+=0.01)
			{
				double p5x = s;
				double p5y = pAckermann(px, py, s);

				int pi = (int)(p5x/a5x-b5x);
				int pj = (int)(-p5y/a5y+b5y);

				PolyPaintBox->Canvas->Pixels[pi][pj] = (TColor)clRed;
			}
		}
		break;

	case RAYTRACING:
		{
			box b5 = box(interval(-10,10), interval(-10,10));
			repere R5 = repere(PolyPaintBox->Canvas,b5[1].inf,b5[1].sup,b5[2].inf,b5[2].sup);
			R5.DrawBox(b5,clBtnFace,clBtnFace,bsSolid);		
		}
		break;

	default:
		break;
	}
}
//---------------------------------------------------------------------------

