/***************************************************************************************************************:')

OSCore.h

Include the main Linux and Windows headers. Some Windows types ported to Linux.
Debug and error handling functions, macros and strings.

Fabrice Le Bars

Created: 2007

Version status: Not finished

***************************************************************************************************************:)*/

#ifndef OSCORE_H
#define OSCORE_H

/*

Predefined macros depending on the compiler : 

_ Visual Studio : _MSC_VER

_ Borland C++ Builder : __BORLANDC__	

_ GCC : __GNUC__

*/

#ifdef _MSC_VER
// Disable some Visual Studio warnings.
#	define CRT_SECURE_NO_DEPRECATE
#	define _CRT_SECURE_NO_WARNINGS
// Enable additional features in math.h.
#	define _USE_MATH_DEFINES
#	ifdef _DEBUG
// Enable additional debug features for malloc functions.
// TO FIX
#		define _CRTDBG_MAP_ALLOC
//#		include <stdlib.h>
//#		include <crtdbg.h>
#	endif // _DEBUG
#endif // _MSC_VER

// If you use Visual Studio in debug configuration, call INIT_DEBUG at the
// beginning of the program to enable memory leaks detection.
// If you do not use Visual Studio, INIT_DEBUG will be ignored.
#if defined(_DEBUG) && defined(_MSC_VER)
#	define INIT_DEBUG _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
#else
#	define INIT_DEBUG
#endif // defined(_DEBUG) && defined(_MSC_VER)

//#define _WIN32_WINNT 0x0500

// C Standard Library headers.
#include <assert.h> // Diagnostics.
#include <ctype.h> // Character Class Tests.
#include <errno.h> // Error Codes Reported by (Some) Library Functions.
#include <float.h> // Implementation-defined Floating-Point Limits.
#include <limits.h> // Implementation-defined Limits.
#include <locale.h> // Locale-specific Information.
#include <math.h> // Mathematical Functions.
#include <setjmp.h> // Non-local Jumps.
#include <signal.h> // Signals.
#include <stdarg.h> // Variable Argument Lists.
#include <stddef.h> // Definitions of General Use.
#include <stdio.h> // Input and Output.
#include <stdlib.h> // Utility functions.
#include <string.h> // String functions.
#include <time.h> // Time and Date functions.

/*
// C99 headers. Some headers are not supported by all the compilers.
#include <complex.h> // Complex arithmetic.
#include <fenv.h> // IEEE-style floating-point arithmetic.
#include <inttypes.h> // Integer types.
#include <iso646.h> // ISO 646 variant character sets.
#include <stdbool.h> // Boolean type name and constants.
#include <stdint.h> // Integer types with size constraints.
#include <tgmath.h> // Type-generic math functions.
#include <wchar.h> // Wide streams and several kinds of strings.
#include <wctype.h> // Wide characters.
*/

#include <fcntl.h>

#ifdef _WIN32
// Prevent Winsock.h from being included by the Windows.h header.
// This must be done if we plan to include Winsock2.h in another files.
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif // WIN32_LEAN_AND_MEAN
#include <windows.h>
#else 
#include <unistd.h>
#include <sys/time.h>
#endif // _WIN32

#ifndef EXTERN_C
#ifdef __cplusplus
#define EXTERN_C extern "C"
#else
#define EXTERN_C
#endif // __cplusplus
#endif // EXTERN_C

#ifndef __BORLANDC__
#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif // max

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif // min
#endif // __BORLANDC__

#ifndef _WIN32
typedef char				CHAR;
typedef unsigned char		UCHAR;
typedef short				SHORT;
typedef unsigned short		USHORT;
typedef int					INT;
typedef unsigned int		UINT;
typedef long				LONG;
typedef unsigned long		ULONG;
typedef float				FLOAT;
#endif // _WIN32
typedef double				DOUBLE;

#ifndef _WIN32
//#define UNREFERENCED_PARAMETER(P)          (P)
//#define DBG_UNREFERENCED_PARAMETER(P)      (P)
//#define DBG_UNREFERENCED_LOCAL_VARIABLE(V) (V)
#define ZeroMemory(Destination,Length) memset((Destination),0,(Length))
typedef void* HANDLE;
typedef unsigned int		u_int;
typedef unsigned long       DWORD;
typedef int                 BOOL;
typedef unsigned char       BYTE;
typedef unsigned short      WORD;
#endif // _WIN32

typedef unsigned char uint8;

typedef unsigned short uint16;

typedef unsigned long uint32;

#ifndef __BORLANDC__
typedef unsigned long long uint64;
#endif // __BORLANDC__

typedef union
{
	uint16 v;  
	uint8 c[2];
} uShort;

typedef union
{
	float v;  
	uint8 c[4];
} uFloat;

// For Windows, debug is activated if _DEBUG is defined. 
// In this case, default is to only activate errors for OSUtils.
#if defined(_DEBUG) && !defined(_DEBUG_MESSAGES) && !defined(_DEBUG_WARNINGS) && !defined(_DEBUG_ERRORS)
#define _DEBUG_ERRORS
#endif // defined(_DEBUG) && !defined(_DEBUG_MESSAGES) && !defined(_DEBUG_WARNINGS) && !defined(_DEBUG_ERRORS)

// If _DEBUG_MESSAGES is defined, messages, warnings and errors are activated.
// If _DEBUG_WARNINGS is defined, only warnings and errors are activated.
// If _DEBUG_ERRORS is defined, only errors are activated.
#ifdef _DEBUG_MESSAGES
#	define _DEBUG_WARNINGS
#	define _DEBUG_ERRORS
#else
#	ifdef _DEBUG_WARNINGS
#		define _DEBUG_ERRORS
#	endif // _DEBUG_WARNINGS
#endif // _DEBUG_MESSAGES

#if defined(_DEBUG_MESSAGES) || defined(_DEBUG_WARNINGS) || defined(_DEBUG_ERRORS)

// Default is to print on stdout/err
#if !defined(_DEBUG_DISPLAY) && !defined(_DEBUG_FILE)
#define _DEBUG_DISPLAY
#endif // !defined(_DEBUG_DISPLAY) && !defined(_DEBUG_FILE)

#endif // defined(_DEBUG_MESSAGES) || defined(_DEBUG_WARNINGS) || defined(_DEBUG_ERRORS) 

#ifdef _DEBUG_DISPLAY
#	define PRINT_DEBUG_MESSAGE printf
#	define PRINT_DEBUG_WARNING printf
#	define PRINT_DEBUG_ERROR fprintf_stderr
#else
#	ifdef _DEBUG_FILE
#		define PRINT_DEBUG_MESSAGE fprintf_messages_file
#		define PRINT_DEBUG_WARNING fprintf_warnings_file
#		define PRINT_DEBUG_ERROR fprintf_errors_file
#	else
#		define PRINT_DEBUG_MESSAGE
#		define PRINT_DEBUG_WARNING
#		define PRINT_DEBUG_ERROR
#	endif // _DEBUG_FILE
#endif // _DEBUG_DISPLAY

#ifdef _DEBUG_DISPLAY
__inline void fprintf_stderr(const char * _Format, ...)
{
	va_list VaList;

	va_start(VaList, _Format);
	vfprintf(stderr, _Format, VaList);
	va_end(VaList);
}
#endif // _DEBUG_DISPLAY

#ifdef _DEBUG_FILE

#define MESSAGES_FILENAME "messages.txt"
#define WARNINGS_FILENAME "warnings.txt"
#define ERRORS_FILENAME "errors.txt"

__inline void fprintf_messages_file(const char * _Format, ...)
{
	FILE* f;
	va_list VaList;

	va_start(VaList, _Format);
	f = fopen(MESSAGES_FILENAME, "a");
	vfprintf(f, _Format, VaList);
	fclose(f);
	va_end(VaList);
}

__inline void fprintf_warnings_file(const char * _Format, ...)
{
	FILE* f;
	va_list VaList;

	va_start(VaList, _Format);
	f = fopen(WARNINGS_FILENAME, "a");
	vfprintf(f, _Format, VaList);
	fclose(f);
	va_end(VaList);
}

__inline void fprintf_errors_file(const char * _Format, ...)
{
	FILE* f;
	va_list VaList;

	va_start(VaList, _Format);
	f = fopen(ERRORS_FILENAME, "a");
	vfprintf(f, _Format, VaList);
	fclose(f);
	va_end(VaList);
}
#endif // _DEBUG_FILE

/*
Debug macros specific to OSUtils.
*/
#ifdef _DEBUG_MESSAGES
#	define _DEBUG_MESSAGES_OSUTILS
#endif // _DEBUG_MESSAGES

#ifdef _DEBUG_WARNINGS
#	define _DEBUG_WARNINGS_OSUTILS
#endif // _DEBUG_WARNINGS

#ifdef _DEBUG_ERRORS
#	define _DEBUG_ERRORS_OSUTILS
#endif // _DEBUG_ERRORS

#ifdef _DEBUG_MESSAGES_OSUTILS
#	define _DEBUG_MESSAGES_OSCORE
#	define _DEBUG_MESSAGES_OSTIME
#	define _DEBUG_MESSAGES_OSMISC
#endif // _DEBUG_MESSAGES_OSUTILS

#ifdef _DEBUG_WARNINGS_OSUTILS
#	define _DEBUG_WARNINGS_OSCORE
#	define _DEBUG_WARNINGS_OSTIME
#	define _DEBUG_WARNINGS_OSMISC
#endif // _DEBUG_WARNINGS_OSUTILS

#ifdef _DEBUG_ERRORS_OSUTILS
#	define _DEBUG_ERRORS_OSCORE
#	define _DEBUG_ERRORS_OSTIME
#	define _DEBUG_ERRORS_OSMISC
#endif // _DEBUG_ERRORS_OSUTILS

#ifdef _DEBUG_MESSAGES_OSUTILS
#	define PRINT_DEBUG_MESSAGE_OSUTILS PRINT_DEBUG_MESSAGE
#else
#	define PRINT_DEBUG_MESSAGE_OSUTILS
#endif // _DEBUG_MESSAGES_OSUTILS

#ifdef _DEBUG_WARNINGS_OSUTILS
#	define PRINT_DEBUG_WARNING_OSUTILS PRINT_DEBUG_WARNING
#else
#	define PRINT_DEBUG_WARNING_OSUTILS
#endif // _DEBUG_WARNINGS_OSUTILS

#ifdef _DEBUG_ERRORS_OSUTILS
#	define PRINT_DEBUG_ERROR_OSUTILS PRINT_DEBUG_ERROR
#else
#	define PRINT_DEBUG_ERROR_OSUTILS
#endif // _DEBUG_ERRORS_OSUTILS

/*
Debug macros specific to OSCore.
*/
#ifdef _DEBUG_MESSAGES_OSCORE
#	define PRINT_DEBUG_MESSAGE_OSCORE PRINT_DEBUG_MESSAGE
#else
#	define PRINT_DEBUG_MESSAGE_OSCORE
#endif // _DEBUG_MESSAGES_OSCORE

#ifdef _DEBUG_WARNINGS_OSCORE
#	define PRINT_DEBUG_WARNING_OSCORE PRINT_DEBUG_WARNING
#else
#	define PRINT_DEBUG_WARNING_OSCORE
#endif // _DEBUG_WARNINGS_OSCORE

#ifdef _DEBUG_ERRORS_OSCORE
#	define PRINT_DEBUG_ERROR_OSCORE PRINT_DEBUG_ERROR
#else
#	define PRINT_DEBUG_ERROR_OSCORE
#endif // _DEBUG_ERRORS_OSCORE

#endif // OSCORE_H
